package com.canvassoft.Lib;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

public class AdvanceGameInfo
{
	public GameInfo info;
	public GameSetting setting;
	
	public Agent self;
	public Role selfRole;
	public int selfIndex;
	
	public int playerCount;
	
	public int[][] agentProbabilitys;

	public List<Integer> aliveAgents = new ArrayList<Integer>();
	public List<Integer> hungAgents = new ArrayList<Integer>();
	public List<Integer> killedAgents = new ArrayList<Integer>();

	public List<Integer> seerCOInfos = new ArrayList<Integer>();
	public List<Integer> mediumCOInfos = new ArrayList<Integer>();
	public List<Integer> guardCOInfos = new ArrayList<Integer>();

	public List<Integer> seerDieInfos = new ArrayList<Integer>();
	public List<Integer> mediumDieInfos = new ArrayList<Integer>();
	
	public int[][][] seerResultInfos;
	public int[][][] mediumResultInfos;


	public AdvanceGameInfo()
	{
	}
	
	public void init(GameInfo _info, GameSetting _setting)
	{		
		info = _info;
		setting = _setting;
		
		self = info.getAgent();
		selfRole = info.getRole();
		selfIndex = self.getAgentIdx();
		Debug.Log("SelfRole:"+selfRole+" / RoleIndex:"+Define.RoleIndex(selfRole)+" / Id:"+selfIndex);
		
		playerCount = setting.getPlayerNum()+1;
		agentProbabilitys = new int[playerCount][Define.Num];
		
		seerResultInfos = new int[playerCount][playerCount][Define.Num];
		mediumResultInfos = new int[playerCount][playerCount][Define.Num];

		aliveAgents.clear();
		for (Agent agent : info.getAliveAgentList())
		{
			Debug.Log("Alive Agent:"+agent.toString());
			aliveAgents.add(agent.getAgentIdx());
		}
	}

	public Agent indexToAgent(int index)
	{
		for (Agent agent : info.getAgentList())
		{
			if (agent.getAgentIdx() == index)return agent;
		}
		return null;
	}
	
	public void addCO(Agent agent, Role role)
	{
		if (role == Role.SEER)
		{
			Debug.Log(String.format("%d is SEER!", agent.getAgentIdx()));
			if (!seerCOInfos.contains(agent.getAgentIdx()))
			{
				seerCOInfos.add(agent.getAgentIdx());
			}
		}
		else if (role == Role.MEDIUM)
		{
			if (!mediumCOInfos.contains(agent.getAgentIdx()))
			{
				mediumCOInfos.add(agent.getAgentIdx());
			}
		}
		else if (role == Role.BODYGUARD)
		{
			if (!guardCOInfos.contains(agent.getAgentIdx()))
			{
				guardCOInfos.add(agent.getAgentIdx());
			}
		}
	}
	
	public static int SeerTable[][][] = 
	{
		{
			{0, 100,   0, 0,  0,  0}, // 1人
			{0, 100,   0, 0,100,  0}, // 2人
			{0, 100,   0, 0,100,100}, // 3人
		},
		{
			{0, 0, 0, 0,100,  0}, // 1人
			{0, 0, 0, 0,100,100}, // 2人
			{0, 0, 0, 0,100,100}, // 3人
		},
		{
			{0, 100,   0, 0,  0,  0}, // 1人
			{0, 100,   0, 0,100,  0}, // 2人
			{0, 100,   0, 0,100,100}, // 3人
		},
		{
			{0, 100,   0, 0,  0,  0}, // 1人
			{0, 100,   0, 0,100,  0}, // 2人
			{0, 100,   0, 0,100,100}, // 3人
		},
		{
			{0, 100, 0, 0,  0,  0}, // 1人
			{0, 100, 0, 0,  0,100}, // 2人
			{0, 100, 0, 0,  0,100}, // 3人
		},
		{
			{0, 100, 0, 0,   0, 0}, // 1人
			{0, 100, 0, 0, 100, 0}, // 2人
			{0, 100, 0, 0, 100, 0}, // 3人
		}
	};
	
	public static int MediumTable[][][] = 
	{
		{
			{0, 0, 100, 0,  0,  0}, // 1人
			{0, 0, 100, 0,100,  0}, // 2人
			{0, 0, 100, 0,100,100}, // 3人
		},
		{
			{0, 0, 100, 0,  0,  0}, // 1人
			{0, 0, 100, 0,100,  0}, // 2人
			{0, 0, 100, 0,100,100}, // 3人
		},
		{
			{0, 0,   0, 0,100,  0}, // 1人
			{0, 0, 100, 0,100,100}, // 2人
			{0, 0, 100, 0,100,100}, // 3人
		},
		{
			{0, 0, 100, 0,  0,  0}, // 1人
			{0, 0, 100, 0,100,  0}, // 2人
			{0, 0, 100, 0,100,100}, // 3人
		},
		{
			{0, 0, 100, 0, 0,   0}, // 1人
			{0, 0, 100, 0, 0, 100}, // 2人
			{0, 0, 100, 0, 0, 100}, // 3人
		},
		{
			{0, 0, 100, 0,   0, 0}, // 1人
			{0, 0, 100, 0, 100, 0}, // 2人
			{0, 0, 100, 0, 100, 0}, // 3人
		}
	};
	
	public float getRolePercent(int agent_index, Role role)
	{
		int role_index = Define.RoleIndex(role);
		if (agent_index == selfIndex)
		{
			if (selfRole == role)return 1.0f;
			return 0.0f;
		}

		if (agentProbabilitys[agent_index][role_index] == 0)
		{
			//Debug.Log("RolePercenter char is 0");
			return 0;
		}
		
		int total = 0;
		for (int i=0; i<playerCount; i++)
		{
			total += agentProbabilitys[i][role_index];
		}
		
		return (float)agentProbabilitys[agent_index][role_index]/(float)total;
	}


    public void addDivined(Agent srcAgent, Agent targetAgent, Species specie)
    {
    	int src_index = srcAgent.getAgentIdx();
    	int target_index = targetAgent.getAgentIdx();
    	if (specie == Species.WEREWOLF)
    	{
    		seerResultInfos[src_index][target_index][Define.RoleIndex(Role.WEREWOLF)] = 1;
    	}
    	else
    	{
    		seerResultInfos[src_index][target_index][Define.RoleIndex(Role.VILLAGER)] = 1;
    		seerResultInfos[src_index][target_index][Define.RoleIndex(Role.POSSESSED)] = 1;
    	}
    }
    
    public void addInquested(Agent srcAgent, Agent targetAgent, Species specie)
    {
    	int src_index = srcAgent.getAgentIdx();
    	int target_index = targetAgent.getAgentIdx();
    	
       	if (specie == Species.WEREWOLF)
    	{
       		mediumResultInfos[src_index][target_index][Define.RoleIndex(Role.WEREWOLF)] = 1;
    	}
    	else
    	{
    		mediumResultInfos[src_index][target_index][Define.RoleIndex(Role.VILLAGER)] = 1;
    		mediumResultInfos[src_index][target_index][Define.RoleIndex(Role.POSSESSED)] = 1;
    	}
    }

	public int getMostRoleAgent(int role_type)
	{
		return 0;
	}

	public void dumpCO()
	{
		String str_seer = "";
		for (int i : seerCOInfos)
		{
			str_seer += i + ", ";
		}
		if (str_seer != "")
		{
			Debug.Log("// 占いCO ---------------------------");
			Debug.Log(str_seer);
		}
		
		String str_medium = "";
		for (int i : mediumCOInfos)
		{
			str_medium += i + ", ";
		}
		if (str_medium != "")
		{
			Debug.Log("// 霊能CO ---------------------------");
			Debug.Log(str_medium);
		}
		
		String str_guard = "";
		for (int i : guardCOInfos)
		{
			str_guard += i + ", ";
		}
		if (str_guard != "")
		{
			Debug.Log("// 狩人CO ---------------------------");
			Debug.Log(str_guard);
		}
	}
	

    public void dumpResult()
    {
		boolean title = false;
		for (int src = 0; src<playerCount; src++)
		{
			for (int target = 0; target < playerCount; target++)
			{
				for (int i=0; i<Define.Num;i++)
				{
					if (seerResultInfos[src][target][i] == 1)
					{
						if (!title)
						{
							title = true;
							Debug.Log("// 占い結果 -------------------------");
						}
						Debug.Log("srcAgent["+src+"] => targetAgent["+target+"] = Role["+Define.RoleType(i)+"]");
					}
				}
			}
		}

		title = false;
		for (int src = 0; src<playerCount; src++)
		{
			for (int target = 0; target < playerCount; target++)
			{
				for (int i=0; i<Define.Num;i++)
				{
					if (seerResultInfos[src][target][i] == 1)
					{
						if (!title)
						{
							title = true;
							Debug.Log("// 霊媒結果 -------------------------");
						}
						Debug.Log("srcAgent["+src+"] => targetAgent["+target+"] = Role["+Define.RoleType(i)+"]");
					}
				}
			}
		}
    }
}