package com.canvassoft.Lib;

import java.util.Map;
import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import org.aiwolf.client.lib.TemplateTalkFactory;
import org.aiwolf.client.lib.Topic;
import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Judge;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.data.Talk;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

public class Brain
{	
	public int currentTalkIndex;
	public AdvanceGameInfo advanceGameInfo = new AdvanceGameInfo();
	public List<Integer> dieInfos = new ArrayList<Integer>();

	public List<Integer> humansList = new ArrayList<Integer>();
	public List<Integer> wolfsList = new ArrayList<Integer>();
	
	public Agent guardAgent = null;
	public List<Integer> guardsList = new ArrayList<Integer>();

	public int talkTurn;
	private int estimateCount;
	private boolean voteTalked;
	private int currenthunged;

	private Judge judgeSeer;
	private Judge judgeMedium;
	
    public Brain()
    {
		currentTalkIndex = 0;
    }

    public void init(GameInfo info, GameSetting setting)
    {
		advanceGameInfo.init(info, setting);
    }
    
	public void dayStart(GameInfo info)
	{
		talkTurn = 0;
		estimateCount = 0;
		voteTalked = false;
		
		currenthunged = -1;
		Agent executed = info.getExecutedAgent();
		if (executed != null)
		{
			currenthunged = executed.getAgentIdx();
			advanceGameInfo.hungAgents.add(executed.getAgentIdx());
			
			if (advanceGameInfo.seerCOInfos.contains(executed))
			{
				advanceGameInfo.seerDieInfos.add(executed.getAgentIdx());
			}
			if (advanceGameInfo.mediumCOInfos.contains(executed))
			{
				advanceGameInfo.mediumDieInfos.add(executed.getAgentIdx());
			}
		}
		else
		{
			if (guardAgent != null)
			{
				guardsList.add(guardAgent.getAgentIdx());
			}
		}
		guardAgent = null;
		
		Agent hung = info.getAttackedAgent();
		if (hung != null)
		{
			advanceGameInfo.killedAgents.add(hung.getAgentIdx());
			if (advanceGameInfo.seerCOInfos.contains(hung))
			{
				advanceGameInfo.seerDieInfos.add(hung.getAgentIdx());
			}
			if (advanceGameInfo.mediumCOInfos.contains(hung))
			{
				advanceGameInfo.mediumDieInfos.add(hung.getAgentIdx());
			}
		}
		
		judgeSeer = null;
		if (advanceGameInfo.selfRole == Role.SEER)
		{
			judgeSeer = info.getDivineResult();
			if (judgeSeer != null)
			{
				if (judgeSeer.getResult() == Species.HUMAN)
				{
					humansList.add(judgeSeer.getTarget().getAgentIdx());
				}
				else
				{
					wolfsList.add(judgeSeer.getTarget().getAgentIdx());
				}
			}
		}
		
		judgeMedium = null;
		if (advanceGameInfo.selfRole == Role.MEDIUM)
		{
			judgeMedium = info.getMediumResult();
			if (judgeMedium != null)
			{
				if (judgeMedium.getResult() == Species.HUMAN)
				{
					humansList.add(judgeMedium.getTarget().getAgentIdx());
				}
				else
				{
					wolfsList.add(judgeMedium.getTarget().getAgentIdx());
				}
			}
		}
	}
	
	public String createTalkEstimate()
	{
		int[] EstimateLists = {
			Define.Seer,
			Define.Medium,
			Define.WereWolf,
			Define.Possessed,
			Define.Villager
		};

		if (EstimateLists.length <= estimateCount)
		{
			return "";
		}

		for (int k=estimateCount; k<EstimateLists.length; k++)
		{
			Role target_role = Define.RoleType(EstimateLists[estimateCount]);
		
			int most_index = -1;
			float most_percent = 0;
			for (int i=0; i<advanceGameInfo.playerCount; i++)
			{
				if (i == advanceGameInfo.selfIndex)continue;
				float percent = advanceGameInfo.getRolePercent(i, target_role);
				if (most_percent < percent)
				{
					most_percent = percent;
					most_index = i;
				}
			}

			estimateCount++;

			if (most_index != -1)
			{
				Agent target_agent = advanceGameInfo.indexToAgent(most_index);
				return TemplateTalkFactory.estimate(target_agent, target_role);
			}
		}
		return "";
	}
	
	public String createTalkPossessedEstimate()
	{
		int[] EstimateLists = {
			Define.Seer,
			Define.Medium,
			Define.WereWolf,
			Define.Villager
		};

		if (EstimateLists.length <= estimateCount)
		{
			return "";
		}

		for (int k=estimateCount; k<EstimateLists.length; k++)
		{
			Role target_role = Define.RoleType(EstimateLists[estimateCount]);
		
			int most_index = -1;
			float most_percent = 0;
			for (int i=0; i<advanceGameInfo.playerCount; i++)
			{
				if (i == advanceGameInfo.selfIndex)continue;
				float percent = advanceGameInfo.getRolePercent(i, target_role);
				if (most_percent < percent)
				{
					most_percent = percent;
					most_index = i;
				}
			}

			estimateCount++;

			if (most_index != -1)
			{
				Agent target_agent = advanceGameInfo.indexToAgent(most_index);
				if (target_role == Role.SEER)target_role = Role.POSSESSED;
				return TemplateTalkFactory.estimate(target_agent, target_role);
			}
		}
		return "";
	}

	public String createTalkVote()
	{
		if (voteTalked)return "";
		
		ArrayList<Agent> voteCandidates = new ArrayList<Agent>();
		voteCandidates.addAll(advanceGameInfo.info.getAliveAgentList());
		voteCandidates.remove(advanceGameInfo.self);
		Agent agent = getMostBlackAgent(voteCandidates, false);
		
		voteTalked = true;
		if (agent != null)
		{
			return TemplateTalkFactory.vote(agent);
		}

		// 発言数が少ない人
		// その他適当な理由で
		// TODO:時間がないので割愛
		return "";
	}
	
	public String createTalkComingout()
	{
		// TODO:時間がないのでとりあえず役職は初日に宣言
		return "";
	}
	
	public String getTalkSeerResult()
	{
		if (judgeSeer == null)return "";
		String result = TemplateTalkFactory.divined(judgeSeer.getTarget(), judgeSeer.getResult());
		judgeSeer = null;
		return result;
	}

	public String getTalkMediumResult()
	{
		if (judgeMedium == null)return "";
		String result = TemplateTalkFactory.inquested(judgeMedium.getTarget(), judgeMedium.getResult());
		judgeMedium = null;
		return result;
	}

	public String talkGuarded()
	{
		// TODO:今回は時間がないので実装しない,だまって殺される
		return "";
	}
		
	public Agent getSeerAgent(ArrayList<Agent> divineCandidates)
	{
		Agent agent = getMostBlackAgent(divineCandidates, true);
		return agent;
	}

	public Agent getGuardAgent(List<Agent> alives)
	{
		int seer_num = advanceGameInfo.seerCOInfos.size();
		int medium_num = advanceGameInfo.mediumCOInfos.size();
		
		int seer_die_num = advanceGameInfo.seerDieInfos.size();
		int medium_die_num = advanceGameInfo.mediumDieInfos.size();
		
		if (seer_num + medium_num == 1 && seer_die_num + medium_die_num == 0)
		{
			if (seer_num == 1)
			{
				return advanceGameInfo.indexToAgent(advanceGameInfo.seerCOInfos.get(0));
			}
			else
			{
				return advanceGameInfo.indexToAgent(advanceGameInfo.mediumCOInfos.get(0));
			}
		}
		else if (seer_num + medium_num == 2)
		{
			if (seer_die_num + medium_die_num == 0)
			{
				List<Agent> agents = new ArrayList<Agent>();
				for (int i : advanceGameInfo.seerCOInfos)
				{
					agents.add(advanceGameInfo.indexToAgent(i));
				}
				for (int i : advanceGameInfo.mediumCOInfos)
				{
					agents.add(advanceGameInfo.indexToAgent(i));
				}
				return getMostWhiteAgent(agents, true);
			}
		}
		else if (seer_num + medium_num == 3)
		{
			if (seer_die_num + medium_die_num == 0)
			{
				if (seer_num == 1)return advanceGameInfo.indexToAgent(advanceGameInfo.seerCOInfos.get(0));
				else if (medium_num == 1)return advanceGameInfo.indexToAgent(advanceGameInfo.mediumCOInfos.get(0));
			}
			else if ((seer_num == 2 && seer_die_num == 1) || (medium_num == 2 && medium_die_num == 1))
			{
				if (seer_num == 1)return advanceGameInfo.indexToAgent(advanceGameInfo.seerCOInfos.get(0));
				else if (medium_num == 1)return advanceGameInfo.indexToAgent(advanceGameInfo.mediumCOInfos.get(0));
			}
			else if ((seer_num == 2 && seer_die_num == 0) || (medium_num == 2 && medium_die_num == 0))
			{
				List<Agent> agents = new ArrayList<Agent>();
				for (int i : advanceGameInfo.seerCOInfos)
				{
					if (advanceGameInfo.seerDieInfos.contains(i))continue;
					agents.add(advanceGameInfo.indexToAgent(i));
				}
				for (int i : advanceGameInfo.mediumCOInfos)
				{
					if (advanceGameInfo.mediumDieInfos.contains(i))continue;
					agents.add(advanceGameInfo.indexToAgent(i));
				}
				return getMostWhiteAgent(agents, true);
			}
		}
				
		return getMostWhiteAgent(alives, true);
	}
	
	public Agent createTalkAttack()
	{
		return null;
	}
	
	public Agent getVoteAgent(ArrayList<Agent> voteCandidates)
	{
		Agent agent = getMostBlackAgent(voteCandidates, true);
		return agent;

		// TODO : 時間がないので実装しない
		// 確率的に割り出せなかったら発言数が少ない人/の他適当な理由で吊る
	}

	public Agent getWolfVoteAgent(ArrayList<Agent> voteCandidates)
	{
		Agent agent = getMostBlackAgent(voteCandidates, false);
		if (agent == null)
		{
		}
		return agent;
	}

	
	public void update(GameInfo latestDayGameInfo)
	{
		talkTurn++;
		List<Talk> talks = latestDayGameInfo.getTalkList();
		int count = talks.size();
		for (int i=currentTalkIndex; i<count; i++)
		{
			Talk talk = talks.get(i);
			if (talk == null)continue;

			Utterance utterance = new Utterance(talk.getContent());
			Topic topic = utterance.getTopic();
			if (topic == Topic.COMINGOUT)
			{
				Debug.Log("Think => CO:"+talk.getAgent()+" / Role:"+utterance.getRole().toString());
				advanceGameInfo.addCO(talk.getAgent(), utterance.getRole());
			}
			else if(topic == Topic.DIVINED)
			{
				System.out.println("// Think => "+talk.getAgent()+"占った相手 =>"+utterance.getTarget()+" / Role:"+utterance.getResult().toString());				
				advanceGameInfo.addDivined(talk.getAgent(), utterance.getTarget(), utterance.getResult());
			}
			else if (topic == Topic.INQUESTED)
			{
				System.out.println("// Think => "+talk.getAgent()+"霊媒相手 =>"+utterance.getTarget()+" / Role:"+utterance.getResult().toString());								
				advanceGameInfo.addInquested(talk.getAgent(), utterance.getTarget(), utterance.getResult());
			}
			else if (topic == Topic.ESTIMATE)
			{
			}
		}
		currentTalkIndex = count;
		calc();
	}
	
    public void calc()
    {
		for (int i=0; i<advanceGameInfo.playerCount; i++)
		{
			for (int n=0; n<Define.Num; n++)
			{
				advanceGameInfo.agentProbabilitys[i][n] = 0;
			}
		}
    	calcCo();
    	calcGuess();
    	calcDieThink();
    	calcRole();
    }

	private void calcRole()
    {
    	if (advanceGameInfo.selfRole == Role.SEER)
    	{
    		// TODO : 作り方に大きな修正が必要なので時間切れ
    	}
    	else if (advanceGameInfo.selfRole == Role.MEDIUM)
    	{
    		// TODO : 作り方に大きな修正が必要なので時間切れ
    	}
    }
    
    private void calcCo()
    {
		int self_role = Define.RoleIndex(advanceGameInfo.selfRole);

		int seer_num = advanceGameInfo.seerCOInfos.size();
		int medium_num = advanceGameInfo.mediumCOInfos.size();
		int all_num = medium_num + seer_num;
		Debug.Log("Seer:"+seer_num+" / Medium:"+medium_num+" / All:"+all_num);

		if (seer_num > 0)
		{
			if (advanceGameInfo.selfRole == Role.SEER)
			{
				advanceGameInfo.agentProbabilitys[advanceGameInfo.selfIndex][Define.Seer] = 999;
				for (int target_index : advanceGameInfo.seerCOInfos)
				{
					if (target_index == advanceGameInfo.selfIndex)continue;
					
					if (wolfsList.contains(target_index))
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] = 999;
					}
					else if (humansList.contains(target_index))
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] = 999;
					}
					else
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 200;
						advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100;
					}
				}
			}
			else
			{
				if (seer_num>3)seer_num = 3;
			
				for (int target_index : advanceGameInfo.seerCOInfos)
				{
					for (int i=0; i<Define.Num; i++)
					{
						advanceGameInfo.agentProbabilitys[target_index][i] += advanceGameInfo.SeerTable[self_role][seer_num-1][i];					
					}
				}
			}
		}
		
		if (medium_num > 0)
		{
			if (advanceGameInfo.selfRole == Role.MEDIUM)
			{
				advanceGameInfo.agentProbabilitys[advanceGameInfo.selfIndex][Define.Medium] = 999;
				for (int target_index : advanceGameInfo.mediumCOInfos)
				{
					if (target_index == advanceGameInfo.selfIndex)continue;
					if (wolfsList.contains(target_index))
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] = 999;
					}
					else if (humansList.contains(target_index))
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] = 999;
					}
					else
					{
						advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 200;
						advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100;
					}
				}
			}
			else
			{
				if (medium_num > 3)medium_num = 3;
				for (int target_index : advanceGameInfo.mediumCOInfos)
				{
					if (target_index == advanceGameInfo.selfIndex)continue;
					for (int i=0; i<Define.Num; i++)
					{
						advanceGameInfo.agentProbabilitys[target_index][i] += advanceGameInfo.MediumTable[self_role][medium_num-1][i];					
					}
				}
			}
		}
		
		if (all_num >= 4)
		{
			if (advanceGameInfo.mediumCOInfos.size() >= 2)
			{
				for (int target_index : advanceGameInfo.mediumCOInfos)
				{
					if (target_index == advanceGameInfo.selfIndex)continue;
					advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 100;
					advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100;
				}
			}
			if (advanceGameInfo.seerCOInfos.size() >= 2)
			{
				for (int target_index : advanceGameInfo.seerCOInfos)
				{
					if (target_index == advanceGameInfo.selfIndex)continue;
					advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 100;
					advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100;
				}
			}
		}

		if (advanceGameInfo.guardCOInfos.size() > 0)
		{
			for (int gurad_index : advanceGameInfo.guardCOInfos)
			{
			}
		}
    }
    
    private void calcGuess()
    {
		for (int src_index : advanceGameInfo.seerCOInfos)
		{
			float seer_per = advanceGameInfo.getRolePercent(src_index, Role.SEER);
			if (seer_per == 0.0f)continue;

			if (advanceGameInfo.seerResultInfos[src_index][advanceGameInfo.selfIndex][Define.WereWolf] == 1)
			{
				advanceGameInfo.agentProbabilitys[src_index][Define.Villager] = 0;
				advanceGameInfo.agentProbabilitys[src_index][Define.Seer] = 0;
				advanceGameInfo.agentProbabilitys[src_index][Define.Possessed] += 200;
				advanceGameInfo.agentProbabilitys[src_index][Define.WereWolf] += 100;
				seer_per = 0.0f;
				continue;
			}

			for (int target_index = 0; target_index < advanceGameInfo.playerCount; target_index++)
			{
				for (int i=0; i<Define.Num; i++)
				{
					if (advanceGameInfo.seerResultInfos[src_index][target_index][i] == 1)
					{
						advanceGameInfo.agentProbabilitys[target_index][i] += (int)(seer_per * 100);
					}
				}
			}
		}

		for (int src_index : advanceGameInfo.mediumCOInfos)
		{
			float medium_per = advanceGameInfo.getRolePercent(src_index, Role.MEDIUM);
			if (medium_per == 0.0f)continue;
			
			for (int target_index = 0; target_index < advanceGameInfo.playerCount; target_index++)
			{
				for (int i=0; i<Define.Num; i++)
				{
					if (advanceGameInfo.mediumResultInfos[src_index][target_index][i] == 1)
					{
						advanceGameInfo.agentProbabilitys[target_index][i] += (int)(medium_per * 100);
					}
				}
			}
		}

		if (advanceGameInfo.seerCOInfos.size() > 1)
		{
			int src_index = advanceGameInfo.seerCOInfos.get(0);
			for (int target_index = 0; target_index < advanceGameInfo.playerCount; target_index++)
			{
				int role_result = getSeerRoleResult(src_index, target_index);
				if (role_result == Define.None)continue;
				
				boolean seered = true;
				boolean same = true;
				for (int other_src_index : advanceGameInfo.seerCOInfos)
				{
					if (other_src_index == src_index)continue;

					int other_role_result = getSeerRoleResult(other_src_index, target_index);
					if (other_role_result == Define.None)
					{
						seered = false;
						break;
					}
					
					if (other_role_result != role_result)
					{
						same = false;
					}
				}

				if (seered)
				{
					if (same)
					{
						if (role_result == Define.WereWolf)
						{
							advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100 * advanceGameInfo.seerCOInfos.size();
						}
						else
						{
							advanceGameInfo.agentProbabilitys[target_index][Define.Villager] += 100 * advanceGameInfo.seerCOInfos.size();
							advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 50 * advanceGameInfo.seerCOInfos.size();
						}
					}
					else
					{
					}
				}
			}
		}

		if (advanceGameInfo.mediumCOInfos.size() > 1)
		{
			int src_index = advanceGameInfo.mediumCOInfos.get(0);
			for (int target_index = 0; target_index < advanceGameInfo.playerCount; target_index++)
			{
				int role_result = getMediumRoleResult(src_index, target_index);
				if (role_result == Define.None)continue;
				
				boolean mediumed = true;
				boolean same = true;
				for (int other_src_index : advanceGameInfo.mediumCOInfos)
				{
					if (other_src_index == src_index)continue;

					int other_role_result = getMediumRoleResult(other_src_index, target_index);
					if (other_role_result == Define.None)
					{
						mediumed = false;
						break;
					}
					
					if (other_role_result != role_result)
					{
						same = false;
					}
				}

				if (mediumed)
				{
					if (same)
					{
						if (role_result == Define.WereWolf)
						{
							advanceGameInfo.agentProbabilitys[target_index][Define.WereWolf] += 100 * advanceGameInfo.seerCOInfos.size();
						}
						else
						{
							advanceGameInfo.agentProbabilitys[target_index][Define.Villager] += 100 * advanceGameInfo.seerCOInfos.size();
							advanceGameInfo.agentProbabilitys[target_index][Define.Possessed] += 50 * advanceGameInfo.seerCOInfos.size();
						}
					}
					else
					{
					}
				}
			}
		}

		if (advanceGameInfo.seerCOInfos.size() >= 2)
		{
			for (int target_index : advanceGameInfo.seerCOInfos)
			{
			}
		}

		if (advanceGameInfo.mediumCOInfos.size() >= 2)
		{
			for (int target_index : advanceGameInfo.mediumCOInfos)
			{
			}
		}
    }
    
    private void calcDieThink()
    {
    	for (int killedIndex : advanceGameInfo.killedAgents)
    	{
    		advanceGameInfo.agentProbabilitys[killedIndex][Define.WereWolf] = 0;
    	}
    	
    	for (int killedIndex : advanceGameInfo.killedAgents)
    	{
    		for (int src=0; src<advanceGameInfo.playerCount; src++)
    		{
    			if (advanceGameInfo.agentProbabilitys[src][Define.Seer] != 0 &&
    				advanceGameInfo.seerResultInfos[src][killedIndex][Define.WereWolf] != 0)
	    		{
	    			advanceGameInfo.agentProbabilitys[src][Define.Seer] = 0;
	    		}
    			
    			if (advanceGameInfo.agentProbabilitys[src][Define.Medium] != 0 &&
        			advanceGameInfo.mediumResultInfos[src][killedIndex][Define.WereWolf] != 0)
    	    	{
    	    		advanceGameInfo.agentProbabilitys[src][Define.Medium] = 0;
    	    	}
	    	}
		}
    }
	
	private boolean seerContains(int src, int target)
	{
		for (int i=0; i<Define.Num; i++)
		{
			if (advanceGameInfo.seerResultInfos[src][target][i] == 1)return true;
		}
		return false;
	}

	private int getSeerRoleResult(int src_index, int target_index)
	{
		for (int i=0; i<Define.Num; i++)
		{
			if (advanceGameInfo.seerResultInfos[src_index][target_index][i] == 1)return i;
		}
		return Define.None;
	}

	int getMediumRoleResult(int src_index, int target_index)
	{
		for (int i=0; i<Define.Num; i++)
		{
			if (advanceGameInfo.mediumResultInfos[src_index][target_index][i] == 1)return i;
		}
		return Define.None;
	}
	
	public Agent getMostWhiteAgent(List<Agent> agents, boolean randam)
	{
		if (agents == null || agents.size() == 0)return null;
		
		int most_index = -1;
		float most_percent = 0;
		for (Agent agent : agents)
		{
			int i = agent.getAgentIdx();
			if (i == advanceGameInfo.selfIndex)continue;
			if (advanceGameInfo.aliveAgents.contains(i))
			{
				float percent = advanceGameInfo.getRolePercent(i, Role.VILLAGER) 
							  + advanceGameInfo.getRolePercent(i,  Role.SEER)
							  + advanceGameInfo.getRolePercent(i,  Role.MEDIUM)
							  + advanceGameInfo.getRolePercent(i,  Role.BODYGUARD);
				if (most_percent < percent)
				{
					most_percent = percent;
					most_index = i;
				}
			}
		}
		
		if (most_index >= 0)
		{
			Agent agent = advanceGameInfo.indexToAgent(most_index);
			return agent;
		}

		most_percent = 10000;
		for (Agent agent : agents)
		{
			int i = agent.getAgentIdx();
			if (i == advanceGameInfo.selfIndex)continue;
			if (advanceGameInfo.aliveAgents.contains(i))
			{
				float percent = advanceGameInfo.getRolePercent(i, Role.WEREWOLF) 
							  + advanceGameInfo.getRolePercent(i,  Role.POSSESSED);
				if (percent <= most_percent)
				{
					most_percent = percent;
					most_index = i;
				}
			}
		}
		if (most_index >= 0)
		{
			Agent agent = advanceGameInfo.indexToAgent(most_index);
			return agent;
		}
		
		return randam ? randamAgent(agents) : null;
	}
	
	public Agent getMostBlackAgent(List<Agent> agents, boolean randam)
	{
		if (agents == null || agents.size()==0)return null;
		
		int most_index = -1;
		float most_percent = 0;
		for (Agent agent : agents)
		{
			int i = agent.getAgentIdx();
			if (i == advanceGameInfo.selfIndex)continue;
			if (advanceGameInfo.aliveAgents.contains(i))
			{
				float percent = advanceGameInfo.getRolePercent(i, Role.WEREWOLF);
//							  + advanceGameInfo.getRolePercent(i,  Role.POSSESSED) * 0.25f;
				if (most_percent < percent)
				{
					most_percent = percent;
					most_index = i;
				}
			}
		}
		
		if (most_index >= 0)
		{
			Agent agent = advanceGameInfo.indexToAgent(most_index);
			return agent;
		}
		
		if (!randam)return null;
				
		List<Agent> randam_agents = new ArrayList<Agent>();
		for (Agent agent : agents)
		{
			int i = agent.getAgentIdx();
			if (i == advanceGameInfo.selfIndex)continue;
			if (advanceGameInfo.aliveAgents.contains(i))
			{
				float percent = advanceGameInfo.getRolePercent(i, Role.VILLAGER)
							  + advanceGameInfo.getRolePercent(i, Role.SEER)
							  + advanceGameInfo.getRolePercent(i, Role.MEDIUM)
							  - advanceGameInfo.getRolePercent(i,  Role.POSSESSED)
							  - advanceGameInfo.getRolePercent(i,  Role.WEREWOLF);
				if (percent <= 0)
				{
					randam_agents.add(agent);
				}
			}
		}
		
		if (randam_agents.size() > 0)
		{
			return randamAgent(randam_agents);
		}
		else
		{
			return randamAgent(agents);
		}
	}

	public Agent randamAgent(List<Agent> agents)
	{
		if (agents == null || agents.size() == 0)return null;
		if (agents.size() == 1)return agents.get(0);
		while(true)
		{
			int num = new Random().nextInt(agents.size());
			if (agents.get(num) == advanceGameInfo.self)continue;
			return agents.get(num);
		}
	}

	public int randamAgentIndex(List<Integer> agents)
	{
		if (agents == null || agents.size() == 0)return 0;
		if (agents.size() == 1)return agents.get(0);
		
		while(true)
		{
			int num = new Random().nextInt(agents.size());
			if (agents.get(num) == advanceGameInfo.selfIndex)continue;
			return agents.get(num);
		}
	}
}